import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.sound.sampled.*;
import java.util.*;

/**
 * Analyse audio signal and display its spectrum
 * @author Yang Yili
 */
public class SignalAnalyser extends JFrame
{
    /**
     * Create a new SignalAnalyser
     */
    public SignalAnalyser(RecordPlay rp, String fn)
    {
	    // draw interface
		setTitle("Signal Analyser " + fn);
	
		Container cp = getContentPane();
	
		cp.setLayout(new BorderLayout());
		graph.setBackground(Color.white);
		scroller = new JScrollPane(graph);
	
		cp.add("Center", scroller);
	
		double frameRate = 0.0;
		double sampleRate = 0.0;
		double time = 0.0;
		long frameLength = (long)0.0;
		AudioInputStream audioInputStream;
		File file = new File(fn);
		try{
	    audioInputStream = AudioSystem.getAudioInputStream(file);
	    frameLength = audioInputStream.getFrameLength();
	    AudioFileFormat aff = AudioSystem.getAudioFileFormat(file);
	    AudioFormat format = audioInputStream.getFormat();
	    sampleRate = format.getSampleRate();
	    frameRate = format.getFrameRate();
		time = (double)frameLength / frameRate;
		FastFourierTransform fft = new FastFourierTransform();
		end = (int)frameLength - 1;
		Reader read = new Reader("file:"+fn, begin, end);
		graph.draw(fft, sampleRate, time, 1, read);
		
		}catch (Exception exception){
		    	JOptionPane.showMessageDialog(null, 
		    	"Can`t read the file.","", JOptionPane.ERROR_MESSAGE); 
		}
	}
	
	/**
     * Create a new SignalAnalyser
     */
	public SignalAnalyser(String file1, String file2)
    {
		setTitle("Signal Analyser");
	
		Container cp = getContentPane();
	
		cp.setLayout(new BorderLayout());
		graph.setBackground(Color.white);
		scroller = new JScrollPane(graph);
	
		cp.add("Center", scroller);
	
		double frameRate = 0.0;
		double sampleRate = 0.0;
		double time1 = 0.0;
		double time2 = 0.0;
		long frameLength = (long)0.0;
		AudioInputStream audioInputStream;
		File file = new File(file1);
		try{
	    audioInputStream = AudioSystem.getAudioInputStream(file);
	    frameLength = audioInputStream.getFrameLength();
	    AudioFileFormat aff = AudioSystem.getAudioFileFormat(file);
	    AudioFormat format = audioInputStream.getFormat();
	    sampleRate = format.getSampleRate();
	    frameRate = format.getFrameRate();
		end = (int)frameLength - 1;
		time1 = (double)frameLength / frameRate;
		Reader read1 = new Reader("file:"+file1, begin, end);
		
	    file = new File(file2);
	    audioInputStream = AudioSystem.getAudioInputStream(file);
	    frameLength = audioInputStream.getFrameLength();
	    aff = AudioSystem.getAudioFileFormat(file);
	    format = audioInputStream.getFormat();
	    sampleRate = format.getSampleRate();
	    frameRate = format.getFrameRate();
		time2 = (double)frameLength / frameRate;
		FastFourierTransform fft = new FastFourierTransform();
		end = (int)frameLength - 1;
		Reader read2 = new Reader("file:"+file2, begin, end);
		graph.draw(fft, sampleRate, time1, time1, time2, 2, read1, read2);		    
		}catch (Exception exception){
			exception.printStackTrace();
	    	JOptionPane.showMessageDialog(null, 
	    	"Can`t read the file.","", JOptionPane.ERROR_MESSAGE); 
		}
	}/* method constructor */
    
    private RecordPlay recordPlay;
    private GraphPanel graph = new GraphPanel();
    private JScrollPane scroller;
    private Vector spectrum = new Vector();
    private Vector spectrum1 = new Vector();
    private Vector spectrum2 = new Vector();
    private int begin = 0;
    private int end = 0;
}










